import * as ts from "typescript";
import { IClassDiagramOptions } from "./classDiagramOptions";
import { Delinter } from "./delint";
import { IParseOptions } from "./parseOptions";
import * as uml from "./uml/index";
export declare class TypeScriptUml {
    /**
     * Parse the source files in a TypeScript project
     *
     * @static
     * @param {string} rootPath Project root path, if tsConfigPath is not defined, the tsconfig.json file
     * will be searched in this directory
     * @param {string} [tsConfigPath] (Optional) Path to tsconfig.json file
     * @returns {uml.CodeModel} The parse results
     *
     * @memberOf TypeScriptUml
     */
    static parseProject(rootPath: string, options?: IParseOptions): uml.CodeModel;
    /**
     * Parse a single TypeScript source file
     *
     * @static
     * @param {string} fileName Source file to parse
     * @param {ts.ScriptTarget} target TypeScript compiler script target
     * @param {Delinter} [delinter] (Optional) Delinter instance to use for delinting
     * @returns {uml.CodeModel} The parse results
     *
     * @memberOf TypeScriptUml
     */
    static parseFile(fileName: string, target: ts.ScriptTarget, sourceText?: string, delinter?: Delinter): uml.CodeModel;
    /**
     * Generate a uml class diagram from a uml CodeModel description.
     *
     * @static
     * @param {uml.CodeModel} codeModel Uml CodeModel description
     * @param {ITypeScriptUmlOptions} [options] Options
     * @returns {string} Class diagram formatted according to the specified options
     *
     * @memberOf TypeScriptUml
     */
    static generateClassDiagram(codeModel: uml.CodeModel, options?: IClassDiagramOptions): string;
    private static _defaultFormatDiagnosticsHost;
    /**
     * Fill options object with default values for unspecified properties.
     *
     * @private
     * @static
     * @param {IParseOptions} [options] Options object
     * @returns {IParseOptions} Options object with default values for unspecified properties
     *
     * @memberOf TypeScriptUml
     */
    private static _setDefaultParseOptions(options?);
    /**
     * Find and parse the tsconfig.json file for the project in the searchPath.
     *
     * @private
     * @static
     * @param {string} searchPath Base search path to look for the tsconfig
     * @param {string} options Path of the tsconfig.json file to parse (optional)
     * @returns {ts.ParsedCommandLine} The parse results
     *
     * @memberOf TypeScriptUml
     */
    private static _readTsconfig(searchPath, options);
    private static _createIncludeCodeModel(codeModel, include, exclude);
    private static _createExcludeCodeModel(codeModel, exclude);
    private static _isIncluded(object, include);
    private static _isExcluded(object, exclude);
    private constructor();
}
