"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var ts = require("typescript");
var delint_1 = require("./delint");
var formatter = require("./formatter/index");
var uml = require("./uml/index");
var TypeScriptUml = (function () {
    function TypeScriptUml() {
        // Static class, don't allow instantiation
    }
    /**
     * Parse the source files in a TypeScript project
     *
     * @static
     * @param {string} rootPath Project root path, if tsConfigPath is not defined, the tsconfig.json file
     * will be searched in this directory
     * @param {string} [tsConfigPath] (Optional) Path to tsconfig.json file
     * @returns {uml.CodeModel} The parse results
     *
     * @memberOf TypeScriptUml
     */
    TypeScriptUml.parseProject = function (rootPath, options) {
        var _options = this._setDefaultParseOptions(options);
        var tsConfig = this._readTsconfig(rootPath, _options);
        var delinter = new delint_1.Delinter();
        for (var _i = 0, _a = tsConfig.fileNames; _i < _a.length; _i++) {
            var f = _a[_i];
            this.parseFile(f, tsConfig.options.target, undefined, delinter);
        }
        return delinter.umlCodeModel;
    };
    /**
     * Parse a single TypeScript source file
     *
     * @static
     * @param {string} fileName Source file to parse
     * @param {ts.ScriptTarget} target TypeScript compiler script target
     * @param {Delinter} [delinter] (Optional) Delinter instance to use for delinting
     * @returns {uml.CodeModel} The parse results
     *
     * @memberOf TypeScriptUml
     */
    TypeScriptUml.parseFile = function (fileName, target, sourceText, delinter) {
        if (!delinter) {
            delinter = new delint_1.Delinter();
        }
        if (!sourceText) {
            sourceText = fs_1.readFileSync(fileName).toString();
        }
        var sourceFile = ts.createSourceFile(fileName, sourceText, target, /*setParentNodes */ true);
        delinter.parse(sourceFile);
        return delinter.umlCodeModel;
    };
    /**
     * Generate a uml class diagram from a uml CodeModel description.
     *
     * @static
     * @param {uml.CodeModel} codeModel Uml CodeModel description
     * @param {ITypeScriptUmlOptions} [options] Options
     * @returns {string} Class diagram formatted according to the specified options
     *
     * @memberOf TypeScriptUml
     */
    TypeScriptUml.generateClassDiagram = function (codeModel, options) {
        var _formatter = null;
        var defaultOptions = {
            formatter: "yuml",
            nodes: {
                exclude: null,
                include: null,
            },
            plantuml: {
                diagramTags: true,
            },
        };
        if (!options) {
            options = {};
        }
        if (!options.hasOwnProperty("formatter")) {
            options.formatter = defaultOptions.formatter;
        }
        if (!options.hasOwnProperty("nodes")) {
            options.nodes = defaultOptions.nodes;
        }
        else {
            if (!options.nodes.hasOwnProperty("include")) {
                options.nodes.include = defaultOptions.nodes.include;
            }
            if (!options.nodes.hasOwnProperty("exclude")) {
                options.nodes.exclude = defaultOptions.nodes.exclude;
            }
        }
        if (!options.hasOwnProperty("plantuml")) {
            options.plantuml = defaultOptions.plantuml;
        }
        else {
            if (!options.plantuml.hasOwnProperty("diagramTags")) {
                options.plantuml.diagramTags = defaultOptions.plantuml.diagramTags;
            }
        }
        var diagramCodeModel = codeModel;
        if (options.nodes.include) {
            // Include nodes
            diagramCodeModel = this._createIncludeCodeModel(codeModel, options.nodes.include, options.nodes.exclude);
        }
        else if (options.nodes.exclude) {
            // Exclude nodes
            diagramCodeModel = this._createExcludeCodeModel(codeModel, options.nodes.exclude);
        }
        switch (options.formatter) {
            case "yuml":
                _formatter = new formatter.YumlFormatter(options);
                break;
            case "plantuml":
                _formatter = new formatter.PlantUMLFormatter(options);
                break;
            default:
                throw new Error("Unknown formatter " + options.formatter);
        }
        return _formatter.generateClassDiagram(diagramCodeModel);
    };
    /**
     * Fill options object with default values for unspecified properties.
     *
     * @private
     * @static
     * @param {IParseOptions} [options] Options object
     * @returns {IParseOptions} Options object with default values for unspecified properties
     *
     * @memberOf TypeScriptUml
     */
    TypeScriptUml._setDefaultParseOptions = function (options) {
        var defaultOptions = {
            exclude: [],
            include: null,
            tsconfig: null,
        };
        if (!options) {
            options = defaultOptions;
        }
        Object.keys(defaultOptions).forEach(function (key) {
            if (!options.hasOwnProperty(key)) {
                options[key] = defaultOptions[key];
            }
        });
        return options;
    };
    /**
     * Find and parse the tsconfig.json file for the project in the searchPath.
     *
     * @private
     * @static
     * @param {string} searchPath Base search path to look for the tsconfig
     * @param {string} options Path of the tsconfig.json file to parse (optional)
     * @returns {ts.ParsedCommandLine} The parse results
     *
     * @memberOf TypeScriptUml
     */
    TypeScriptUml._readTsconfig = function (searchPath, options) {
        var configPath = options.tsconfig ? options.tsconfig : ts.findConfigFile(searchPath, fs_1.existsSync);
        var config = ts.readConfigFile(configPath, function (path) {
            return fs_1.readFileSync(path).toString();
        });
        if (config.error) {
            var formattedDiagnostic = ts.formatDiagnostics([config.error], this._defaultFormatDiagnosticsHost);
            throw new Error("Failed to read tsconfig file " + configPath + ": " + formattedDiagnostic);
        }
        if (options.include) {
            config.config.include = options.include;
        }
        if (options.exclude) {
            config.config.exclude = !config.config.exclude ?
                options.exclude : config.config.exclude.concat(options.exclude);
        }
        var parsed = ts.parseJsonConfigFileContent(config.config, ts.sys, searchPath);
        if (parsed.errors.length > 0) {
            var formattedDiagnostic = ts.formatDiagnostics(parsed.errors, this._defaultFormatDiagnosticsHost);
            throw new Error("Failed to parse tsconfig file " + configPath + ": " + formattedDiagnostic);
        }
        return parsed;
    };
    TypeScriptUml._createIncludeCodeModel = function (codeModel, include, exclude) {
        var _this = this;
        var newCodeModel = new uml.CodeModel();
        if (exclude === null || exclude === undefined) {
            exclude = [];
        }
        codeModel.nodes.forEach(function (key, node) {
            if (_this._isIncluded(node, include) && !_this._isExcluded(node, exclude)) {
                newCodeModel.nodes.setValue(key, codeModel.nodes.getValue(key));
            }
        });
        codeModel.associations.forEach(function (association) {
            if (_this._isIncluded(association, include) && !_this._isExcluded(association, exclude)) {
                newCodeModel.associations.add(association);
            }
        });
        codeModel.generalizations.forEach(function (generalization) {
            if (_this._isIncluded(generalization, include) && !_this._isExcluded(generalization, exclude)) {
                newCodeModel.generalizations.add(generalization);
            }
        });
        return newCodeModel;
    };
    TypeScriptUml._createExcludeCodeModel = function (codeModel, exclude) {
        var _this = this;
        var newCodeModel = new uml.CodeModel();
        codeModel.nodes.forEach(function (key, node) {
            if (!_this._isExcluded(node, exclude)) {
                newCodeModel.nodes.setValue(key, codeModel.nodes.getValue(key));
            }
        });
        codeModel.associations.forEach(function (association) {
            if (!_this._isExcluded(association, exclude)) {
                newCodeModel.associations.add(association);
            }
        });
        codeModel.generalizations.forEach(function (generalization) {
            if (!_this._isExcluded(generalization, exclude)) {
                newCodeModel.generalizations.add(generalization);
            }
        });
        return newCodeModel;
    };
    TypeScriptUml._isIncluded = function (object, include) {
        var node = object instanceof uml.Node ? object : null;
        var link = object instanceof uml.Link ? object : null;
        if (node) {
            return include.find(function (value) {
                return node.identifier === value;
            }) !== undefined;
        }
        else if (link) {
            return include.find(function (value) { return value === link.fromName; }) !== undefined &&
                include.find(function (value) { return value === link.toName; }) !== undefined;
        }
    };
    TypeScriptUml._isExcluded = function (object, exclude) {
        var node = object instanceof uml.Node ? object : null;
        var link = object instanceof uml.Link ? object : null;
        return exclude.find(function (value) {
            if (node) {
                return node.identifier === value;
            }
            else if (link) {
                return link.fromName === value || link.toName === value;
            }
        }) !== undefined;
    };
    return TypeScriptUml;
}());
/* istanbul ignore next: code only used by TypeScript API, which is mocked during tests */
TypeScriptUml._defaultFormatDiagnosticsHost = {
    getCanonicalFileName: function (fileName) {
        return ts.sys.useCaseSensitiveFileNames ? fileName : fileName.toLowerCase();
    },
    getCurrentDirectory: function () { return ts.sys.getCurrentDirectory(); },
    getNewLine: function () { return ts.sys.newLine; },
};
exports.TypeScriptUml = TypeScriptUml;
